@@ -1,10 +1,10 @@
 
-set rcsId {$Id: demo,v 1.36 1998/05/03 20:13:54 jfontain Exp $}
+set rcsId {$Id: demo,v 1.39 1998/06/09 21:11:45 jfontain Exp $}
 
-if {[catch {package require stooop 3.5.1}]} {
-set rcsId {$Id: stooop.tcl,v 3.60 1998/04/18 20:43:14 jfontain Exp $}
+if {[catch {package require stooop 3.6}]} {
+set rcsId {$Id: stooop.tcl,v 3.62 1998/06/05 18:29:09 jfontain Exp $}
 
-package provide stooop 3.5.1
+package provide stooop 3.6
 
 catch {rename proc _proc}
 
@@ -319,7 +319,7 @@ _proc ::stooop::memberProcedureDeclaration {fullClass class procedure arguments
             _proc ${fullClass}::$procedure $arguments \
 "::variable {}
 $checkCode
-::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         } else {
             _proc ${fullClass}::_$procedure $arguments \
@@ -331,9 +331,9 @@ $body
 "::variable {}
 $checkCode
 if {!\[::catch {::info body \$${fullClass}::(\$this,_derived)::$procedure}\]} {
-::return \[::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
+::return \[::uplevel \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
 }
-::eval ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         }
     } else {
@@ -652,8 +652,8 @@ class switched {
 }
 }
 
-if {[catch {package require tkpiechart 5.0}]} {
-set rcsId {$Id: pielabel.tcl,v 1.38 1998/05/03 10:50:27 jfontain Exp $}
+if {[catch {package require tkpiechart 5.1}]} {
+set rcsId {$Id: pielabel.tcl,v 1.40 1998/06/07 10:07:30 jfontain Exp $}
 
 class pieLabeler {
 
@@ -665,10 +665,6 @@ class pieLabeler {
 
     proc ~pieLabeler {this} {}
 
-    proc link {this pie} {
-        ::set pieLabeler::($this,pie) $pie
-    }
-
     virtual proc new {this slice args}
 
     virtual proc delete {this label}
@@ -677,10 +673,12 @@ class pieLabeler {
 
     virtual proc selectState {this label {state {}}}
 
-    virtual proc update {this} {}
+    virtual proc update {this left top right bottom}
+
+    virtual proc room {this arrayName}
 
 }
-set rcsId {$Id: perilabel.tcl,v 1.39 1998/05/03 18:15:53 jfontain Exp $}
+set rcsId {$Id: perilabel.tcl,v 1.45 1998/06/09 21:17:52 jfontain Exp $}
 
 class piePeripheralLabeler {
 
@@ -688,6 +686,10 @@ class piePeripheralLabeler {
 
     proc piePeripheralLabeler {this canvas args} pieLabeler {$canvas $args} switched {$args} {
         switched::complete $this
+        ::set piePeripheralLabeler::($this,array) [::new canvasLabelsArray $canvas -justify $switched::($this,-justify)]
+        ::set piePeripheralLabeler::($this,valueWidth)\
+            [font measure $switched::($this,-smallfont) $switched::($this,-widestvaluetext)]
+        ::set piePeripheralLabeler::($this,valueHeight) [font metrics $switched::($this,-smallfont) -ascent]
     }
 
     proc ~piePeripheralLabeler {this} {
@@ -702,11 +704,11 @@ class piePeripheralLabeler {
             [list -justify left left]\
             [list -offset 5 5]\
             [list -smallfont {Helvetica -10} {Helvetica -10}]\
-            [list -xoffset 0 0]\
+            [list -widestvaluetext 0.00 0.00]\
         ]
     }
 
-    foreach option {-bulletwidth -font -justify -offset -smallfont -xoffset} {
+    foreach option {-bulletwidth -font -justify -offset -smallfont -widestvaluetext} {
         proc set$option {this value} "
             if {\$switched::(\$this,complete)} {
                 error {option $option cannot be set dynamically}
@@ -714,32 +716,26 @@ class piePeripheralLabeler {
         "
     }
 
+    proc set-smallfont {this value} {
+        if {$switched::($this,complete)} {
+            error {option -smallfont cannot be set dynamically}
+        }
+    }
+
     proc new {this slice args} {
         ::set canvas $pieLabeler::($this,canvas)
-
         ::set text [$canvas create text 0 0 -font $switched::($this,-smallfont) -tags pieLabeler($this)]
-        ::set box [$canvas bbox $text]
-        ::set smallTextHeight [expr {[lindex $box 3]-[lindex $box 1]}]
-
-        if {![info exists piePeripheralLabeler::($this,array)]} {
-            ::set piePeripheralLabeler::($this,array)\
-                [::new canvasLabelsArray $canvas 0 0 -justify $switched::($this,-justify) -xoffset $switched::($this,-xoffset)]
-            update $this
-        }
-        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas) 0 0 $args\
+        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas) $args\
             [list\
                 -style split -justify $switched::($this,-justify) -bulletwidth $switched::($this,-bulletwidth)\
                 -font $switched::($this,-font)\
             ]\
         ]
         canvasLabelsArray::manage $piePeripheralLabeler::($this,array) $label
-
         $canvas addtag pieLabeler($this) withtag canvasLabelsArray($piePeripheralLabeler::($this,array))
-
         ::set piePeripheralLabeler::($this,textItem,$label) $text
         ::set piePeripheralLabeler::($this,slice,$label) $slice
         ::set piePeripheralLabeler::($this,selected,$label) 0
-
         return $label
     }
 
@@ -801,26 +797,38 @@ class piePeripheralLabeler {
         ::set piePeripheralLabeler::($this,selected,$label) $selected
     }
 
-    proc update {this} {
+    proc update {this left top right bottom} {
         ::set canvas $pieLabeler::($this,canvas)
-        ::set box [$canvas bbox pieGraphics($pieLabeler::($this,pie))]
-::set smallTextHeight 10
         ::set array $piePeripheralLabeler::($this,array)
         foreach {x y} [$canvas coords canvasLabelsArray($array)] {}
-        $canvas move canvasLabelsArray($array) [expr {[lindex $box 0]-$x}]\
-            [expr {[lindex $box 3]+(2*$switched::($this,-offset))+$smallTextHeight-$y}]
-        switched::configure $array -width [expr {[lindex $box 2]-[lindex $box 0]}]
+        $canvas move canvasLabelsArray($array) [expr {$left-$x}] [expr {$bottom-[canvasLabelsArray::height $array]-$y}]
+        switched::configure $array -width [expr {$right-$left}]
         foreach label [canvasLabelsArray::labels $array] {
             position $this $piePeripheralLabeler::($this,textItem,$label) $piePeripheralLabeler::($this,slice,$label)
         }
     }
 
+    proc room {this arrayName} {
+        upvar $arrayName data
+
+        ::set data(left) [expr {$piePeripheralLabeler::($this,valueWidth)+$switched::($this,-offset)}]
+        ::set data(right) $data(left)
+        ::set data(top) [expr {$switched::($this,-offset)+$piePeripheralLabeler::($this,valueHeight)}]
+        ::set box [$pieLabeler::($this,canvas) bbox canvasLabelsArray($piePeripheralLabeler::($this,array))]
+        if {[llength $box]==0} {
+            ::set data(bottom) $data(top)
+        } else {
+            ::set data(bottom) [expr {$data(top)+[lindex $box 3]-[lindex $box 1]}]
+        }
+    }
+
 }
-set rcsId {$Id: boxlabel.tcl,v 1.36 1998/05/03 15:01:20 jfontain Exp $}
+set rcsId {$Id: boxlabel.tcl,v 1.39 1998/06/07 10:20:43 jfontain Exp $}
 
 class pieBoxLabeler {
 
     proc pieBoxLabeler {this canvas args} pieLabeler {$canvas $args} switched {$args} {
+        ::set pieBoxLabeler::($this,array) [::new canvasLabelsArray $canvas]
         switched::complete $this
     }
 
@@ -846,11 +854,7 @@ class pieBoxLabeler {
     }
 
     proc new {this slice args} {
-        if {![info exists pieBoxLabeler::($this,array)]} {
-            ::set pieBoxLabeler::($this,array) [::new canvasLabelsArray $pieLabeler::($this,canvas) 0 0]
-            update $this
-        }
-        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas) 0 0\
+        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas)\
             $args [list -justify $switched::($this,-justify) -font $switched::($this,-font)]\
         ]
         canvasLabelsArray::manage $pieBoxLabeler::($this,array) $label
@@ -882,23 +886,35 @@ class pieBoxLabeler {
         ::set pieBoxLabeler::($this,selected,$label) $selected
     }
 
-    proc update {this} {
+    proc update {this left top right bottom} {
         ::set canvas $pieLabeler::($this,canvas)
-        ::set box [$canvas bbox pieGraphics($pieLabeler::($this,pie))]
         ::set array $pieBoxLabeler::($this,array)
         foreach {x y} [$canvas coords canvasLabelsArray($array)] {}
-        $canvas move canvasLabelsArray($array) [expr {[lindex $box 0]-$x}] [expr {[lindex $box 3]+$switched::($this,-offset)-$y}]
-        switched::configure $array -width [expr {[lindex $box 2]-[lindex $box 0]}]
+        $canvas move canvasLabelsArray($array) [expr {$left-$x}] [expr {$bottom-[canvasLabelsArray::height $array]-$y}]
+        switched::configure $array -width [expr {$right-$left}]
     }
 
+    proc room {this arrayName} {
+        upvar $arrayName data
+
+        ::set data(left) 0
+        ::set data(right) 0
+        ::set data(top) 0
+        ::set box [$pieLabeler::($this,canvas) bbox canvasLabelsArray($pieBoxLabeler::($this,array))]
+        if {[llength $box]==0} {
+            ::set data(bottom) 0
+        } else {
+            ::set data(bottom) [expr {[lindex $box 3]-[lindex $box 1]+$switched::($this,-offset)}]
+        }
+    }
 }
-set rcsId {$Id: canlabel.tcl,v 1.21 1998/05/03 10:52:49 jfontain Exp $}
+set rcsId {$Id: canlabel.tcl,v 1.22 1998/06/07 10:08:13 jfontain Exp $}
 
 class canvasLabel {
 
-    proc canvasLabel {this canvas x y args} switched {$args} {
+    proc canvasLabel {this canvas args} switched {$args} {
         set canvasLabel::($this,canvas) $canvas
-        set canvasLabel::($this,origin) [$canvas create image $x $y -tags canvasLabel($this)]
+        set canvasLabel::($this,origin) [$canvas create image 0 0 -tags canvasLabel($this)]
         set canvasLabel::($this,rectangle) [$canvas create rectangle 0 0 0 0 -tags canvasLabel($this)]
         set canvasLabel::($this,text) [$canvas create text 0 0 -tags canvasLabel($this)]
 
@@ -998,13 +1014,13 @@ class canvasLabel {
     }
 
 }
-set rcsId {$Id: labarray.tcl,v 1.18 1998/05/03 15:00:55 jfontain Exp $}
+set rcsId {$Id: labarray.tcl,v 1.20 1998/06/07 13:47:02 jfontain Exp $}
 
 class canvasLabelsArray {
 
-    proc canvasLabelsArray {this canvas x y args} switched {$args} {
+    proc canvasLabelsArray {this canvas args} switched {$args} {
         set canvasLabelsArray::($this,canvas) $canvas
-        set canvasLabelsArray::($this,origin) [$canvas create image $x $y -tags canvasLabelsArray($this)]
+        set canvasLabelsArray::($this,origin) [$canvas create image 0 0 -tags canvasLabelsArray($this)]
         set canvasLabelsArray::($this,labels) {}
         switched::complete $this
     }
@@ -1017,17 +1033,14 @@ class canvasLabelsArray {
     proc options {this} {
         return [list\
             [list -justify left left]\
-            [list -xoffset 0 0]\
             [list -width 100]\
         ]
     }
 
-    foreach option {-justify -xoffset} {
-        proc set$option {this value} "
-            if {\$switched::(\$this,complete)} {
-                error {option $option cannot be set dynamically}
-            }
-        "
+    proc set-justify {this value} {
+        if {$switched::($this,complete)} {
+            error {option -justify cannot be set dynamically}
+        }
     }
 
     proc set-width {this value} {
@@ -1067,7 +1080,6 @@ class canvasLabelsArray {
         set canvas $canvasLabelsArray::($this,canvas)
 
         foreach {x y} [$canvas coords $canvasLabelsArray::($this,origin)] {}
-        set x [expr {$x+(($index%2?1:-1)*$switched::($this,-xoffset))}]
         set coordinates [$canvas bbox canvasLabel($label)]
         set y [expr {$y+(($index/2)*([lindex $coordinates 3]-[lindex $coordinates 1]))}]
 
@@ -1094,22 +1106,30 @@ class canvasLabelsArray {
         return $canvasLabelsArray::($this,labels)
     }
 
+    proc height {this} {
+        set number [llength $canvasLabelsArray::($this,labels)]
+        if {$number==0} {
+            return 0
+        }
+        set coordinates [$canvasLabelsArray::($this,canvas) bbox canvasLabel([lindex $canvasLabelsArray::($this,labels) 0])]
+        return [expr {(($number+1)/2)*([lindex $coordinates 3]-[lindex $coordinates 1])}]
+    }
+
 }
-set rcsId {$Id: slice.tcl,v 1.35 1998/05/03 18:08:55 jfontain Exp $}
+set rcsId {$Id: slice.tcl,v 1.39 1998/06/07 10:19:25 jfontain Exp $}
 
 
 class slice {
     variable PI 3.14159265358979323846
 }
 
-proc slice::slice {this canvas x y radiusX radiusY start extent args} switched {$args} {
+proc slice::slice {this canvas xRadius yRadius args} switched {$args} {
     set slice::($this,canvas) $canvas
-    set slice::($this,start) 0
-    set slice::($this,radiusX) $radiusX
-    set slice::($this,radiusY) $radiusY
+    set slice::($this,xRadius) $xRadius
+    set slice::($this,yRadius) $yRadius
     switched::complete $this
-    complete $this $x $y
-    update $this $start $extent
+    complete $this
+    update $this
 }
 
 proc slice::~slice {this} {
@@ -1125,6 +1145,7 @@ proc slice::options {this} {
         [list -deletecommand {} {}]\
         [list -height 0 0]\
         [list -scale {1 1} {1 1}]\
+        [list -startandextent {0 0} {0 0}]\
         [list -topcolor {} {}]\
     ]
 }
@@ -1141,7 +1162,22 @@ proc slice::set-deletecommand {this value} {}
 
 proc slice::set-scale {this value} {
     if {$switched::($this,complete)} {
-        update $this $slice::($this,start) $slice::($this,extent)
+        update $this
+    }
+}
+
+proc slice::set-startandextent {this value} {
+    foreach {start extent} $value {}
+    set slice::($this,start) [normalizedAngle $start]
+    if {$extent<0} {
+        set slice::($this,extent) 0
+    } elseif {$extent>=360} {
+        set slice::($this,extent) [expr {360-pow(10,-$::tcl_precision+3)}]
+    } else {
+        set slice::($this,extent) $extent
+    }
+    if {$switched::($this,complete)} {
+        update $this
     }
 }
 
@@ -1155,12 +1191,12 @@ proc slice::normalizedAngle {value} {
     return $value
 }
 
-proc slice::complete {this x y} {
+proc slice::complete {this} {
     set canvas $slice::($this,canvas)
-    set radiusX $slice::($this,radiusX)
-    set radiusY $slice::($this,radiusY)
+    set xRadius $slice::($this,xRadius)
+    set yRadius $slice::($this,yRadius)
     set bottomColor $switched::($this,-bottomcolor)
-    set slice::($this,origin) [$canvas create image -$radiusX -$radiusY -tags slice($this)]
+    set slice::($this,origin) [$canvas create image -$xRadius -$yRadius -tags slice($this)]
     if {$switched::($this,-height)>0} {
         set slice::($this,startBottomArcFill) [$canvas create arc\
             0 0 0 0 -style chord -extent 0 -fill $bottomColor -outline $bottomColor -tags slice($this)\
@@ -1180,28 +1216,23 @@ proc slice::complete {this x y} {
         set slice::($this,endRightLine) [$canvas create line 0 0 0 0 -tags slice($this)]
     }
     set slice::($this,topArc) [$canvas create arc\
-        -$radiusX -$radiusY $radiusX $radiusY -fill $switched::($this,-topcolor) -tags slice($this)\
+        -$xRadius -$yRadius $xRadius $yRadius -fill $switched::($this,-topcolor) -tags slice($this)\
     ]
-    $canvas move slice($this) [expr {$x+$radiusX}] [expr {$y+$radiusY}]
+    $canvas move slice($this) $xRadius $yRadius
 }
 
-proc slice::update {this start extent} {
+proc slice::update {this} {
     set canvas $slice::($this,canvas)
     set coordinates [$canvas coords $slice::($this,origin)]
-    set radiusX $slice::($this,radiusX)
-    set radiusY $slice::($this,radiusY)
-    $canvas coords $slice::($this,origin) -$radiusX -$radiusY
-    $canvas coords $slice::($this,topArc) -$radiusX -$radiusY $radiusX $radiusY
-    set extent [maximum 0 $extent]
-    if {$extent>=360} {
-        set extent [expr {360-pow(10,-$::tcl_precision+3)}]
-    }
-    $canvas itemconfigure $slice::($this,topArc)\
-        -start [set slice::($this,start) [normalizedAngle $start]] -extent [set slice::($this,extent) $extent]
+    set xRadius $slice::($this,xRadius)
+    set yRadius $slice::($this,yRadius)
+    $canvas coords $slice::($this,origin) -$xRadius -$yRadius
+    $canvas coords $slice::($this,topArc) -$xRadius -$yRadius $xRadius $yRadius
+    $canvas itemconfigure $slice::($this,topArc) -start $slice::($this,start) -extent $slice::($this,extent)
     if {$switched::($this,-height)>0} {
         updateBottom $this
     }
-    $canvas move slice($this) [expr {[lindex $coordinates 0]+$radiusX}] [expr {[lindex $coordinates 1]+$radiusY}]
+    $canvas move slice($this) [expr {[lindex $coordinates 0]+$xRadius}] [expr {[lindex $coordinates 1]+$yRadius}]
     eval $canvas scale slice($this) $coordinates $switched::($this,-scale)
 }
 
@@ -1212,34 +1243,34 @@ proc slice::updateBottom {this} {
     set extent $slice::($this,extent)
 
     set canvas $slice::($this,canvas)
-    set radiusX $slice::($this,radiusX)
-    set radiusY $slice::($this,radiusY)
+    set xRadius $slice::($this,xRadius)
+    set yRadius $slice::($this,yRadius)
     set height $switched::($this,-height)
 
     $canvas itemconfigure $slice::($this,startBottomArcFill) -extent 0
-    $canvas coords $slice::($this,startBottomArcFill) -$radiusX -$radiusY $radiusX $radiusY
+    $canvas coords $slice::($this,startBottomArcFill) -$xRadius -$yRadius $xRadius $yRadius
     $canvas move $slice::($this,startBottomArcFill) 0 $height
     $canvas itemconfigure $slice::($this,startBottomArc) -extent 0
-    $canvas coords $slice::($this,startBottomArc) -$radiusX -$radiusY $radiusX $radiusY
+    $canvas coords $slice::($this,startBottomArc) -$xRadius -$yRadius $xRadius $yRadius
     $canvas move $slice::($this,startBottomArc) 0 $height
     $canvas coords $slice::($this,startLeftLine) 0 0 0 0
     $canvas coords $slice::($this,startRightLine) 0 0 0 0
     $canvas itemconfigure $slice::($this,endBottomArcFill) -extent 0
-    $canvas coords $slice::($this,endBottomArcFill) -$radiusX -$radiusY $radiusX $radiusY
+    $canvas coords $slice::($this,endBottomArcFill) -$xRadius -$yRadius $xRadius $yRadius
     $canvas move $slice::($this,endBottomArcFill) 0 $height
     $canvas itemconfigure $slice::($this,endBottomArc) -extent 0
-    $canvas coords $slice::($this,endBottomArc) -$radiusX -$radiusY $radiusX $radiusY
+    $canvas coords $slice::($this,endBottomArc) -$xRadius -$yRadius $xRadius $yRadius
     $canvas move $slice::($this,endBottomArc) 0 $height
     $canvas coords $slice::($this,endLeftLine) 0 0 0 0
     $canvas coords $slice::($this,endRightLine) 0 0 0 0
     $canvas coords $slice::($this,startPolygon) 0 0 0 0 0 0 0 0
     $canvas coords $slice::($this,endPolygon) 0 0 0 0 0 0 0 0
 
-    set startX [expr {$radiusX*cos($start*$PI/180)}]
-    set startY [expr {-$radiusY*sin($start*$PI/180)}]
+    set startX [expr {$xRadius*cos($start*$PI/180)}]
+    set startY [expr {-$yRadius*sin($start*$PI/180)}]
     set end [normalizedAngle [expr {$start+$extent}]]
-    set endX [expr {$radiusX*cos($end*$PI/180)}]
-    set endY [expr {-$radiusY*sin($end*$PI/180)}]
+    set endX [expr {$xRadius*cos($end*$PI/180)}]
+    set endY [expr {-$yRadius*sin($end*$PI/180)}]
 
     set startBottom [expr {$startY+$height}]
     set endBottom [expr {$endY+$height}]
@@ -1257,54 +1288,46 @@ proc slice::updateBottom {this} {
             if {$start<0} {
                 $canvas itemconfigure $slice::($this,startBottomArcFill) -start 0 -extent $start
                 $canvas itemconfigure $slice::($this,startBottomArc) -start 0 -extent $start
-                $canvas coords $slice::($this,startPolygon) $startX $startY $radiusX 0 $radiusX $height $startX $startBottom
+                $canvas coords $slice::($this,startPolygon) $startX $startY $xRadius 0 $xRadius $height $startX $startBottom
                 $canvas coords $slice::($this,startLeftLine) $startX $startY $startX $startBottom
-                $canvas coords $slice::($this,startRightLine) $radiusX 0 $radiusX $height
+                $canvas coords $slice::($this,startRightLine) $xRadius 0 $xRadius $height
 
                 set bottomArcExtent [expr {$end+180}]
                 $canvas itemconfigure $slice::($this,endBottomArcFill) -start -180 -extent $bottomArcExtent
                 $canvas itemconfigure $slice::($this,endBottomArc) -start -180 -extent $bottomArcExtent
-                $canvas coords $slice::($this,endPolygon) -$radiusX 0 $endX $endY $endX $endBottom -$radiusX $height
-                $canvas coords $slice::($this,endLeftLine) -$radiusX 0 -$radiusX $height
+                $canvas coords $slice::($this,endPolygon) -$xRadius 0 $endX $endY $endX $endBottom -$xRadius $height
+                $canvas coords $slice::($this,endLeftLine) -$xRadius 0 -$xRadius $height
                 $canvas coords $slice::($this,endRightLine) $endX $endY $endX $endBottom
             } else {
                 $canvas itemconfigure $slice::($this,startBottomArcFill) -start 0 -extent -180
                 $canvas itemconfigure $slice::($this,startBottomArc) -start 0 -extent -180
-                $canvas coords $slice::($this,startPolygon) -$radiusX 0 $radiusX 0 $radiusX $height -$radiusX $height
-                $canvas coords $slice::($this,startLeftLine) -$radiusX 0 -$radiusX $height
-                $canvas coords $slice::($this,startRightLine) $radiusX 0 $radiusX $height
+                $canvas coords $slice::($this,startPolygon) -$xRadius 0 $xRadius 0 $xRadius $height -$xRadius $height
+                $canvas coords $slice::($this,startLeftLine) -$xRadius 0 -$xRadius $height
+                $canvas coords $slice::($this,startRightLine) $xRadius 0 $xRadius $height
             }
         }
     } else {
         if {$start<0} {
             $canvas itemconfigure $slice::($this,startBottomArcFill) -start 0 -extent $start
             $canvas itemconfigure $slice::($this,startBottomArc) -start 0 -extent $start
-            $canvas coords $slice::($this,startPolygon) $startX $startY $radiusX 0 $radiusX $height $startX $startBottom
+            $canvas coords $slice::($this,startPolygon) $startX $startY $xRadius 0 $xRadius $height $startX $startBottom
             $canvas coords $slice::($this,startLeftLine) $startX $startY $startX $startBottom
-            $canvas coords $slice::($this,startRightLine) $radiusX 0 $radiusX $height
+            $canvas coords $slice::($this,startRightLine) $xRadius 0 $xRadius $height
         } else {
             set bottomArcExtent [expr {$end+180}]
             $canvas itemconfigure $slice::($this,endBottomArcFill) -start -180 -extent $bottomArcExtent
             $canvas itemconfigure $slice::($this,endBottomArc) -start -180 -extent $bottomArcExtent
-            $canvas coords $slice::($this,endPolygon) -$radiusX 0 $endX $endY $endX $endBottom -$radiusX $height
-            $canvas coords $slice::($this,startLeftLine) -$radiusX 0 -$radiusX $height
+            $canvas coords $slice::($this,endPolygon) -$xRadius 0 $endX $endY $endX $endBottom -$xRadius $height
+            $canvas coords $slice::($this,startLeftLine) -$xRadius 0 -$xRadius $height
             $canvas coords $slice::($this,startRightLine) $endX $endY $endX $endBottom
         }
     }
 }
 
-proc slice::position {this start} {
-    update $this $start $slice::($this,extent)
-}
-
 proc slice::rotate {this angle} {
-    if {$angle!=0} {
-        update $this [expr {$slice::($this,start)+$angle}] $slice::($this,extent)
-    }
-}
-
-proc slice::size {this extent} {
-    update $this $slice::($this,start) $extent
+    if {$angle==0} return
+    set slice::($this,start) [normalizedAngle [expr {$slice::($this,start)+$angle}]]
+    update $this
 }
 
 proc slice::data {this arrayName} {
@@ -1313,19 +1336,14 @@ proc slice::data {this arrayName} {
     set data(start) $slice::($this,start)
     set data(extent) $slice::($this,extent)
     foreach {x y} $switched::($this,-scale) {}
-    set data(xRadius) [expr {$x*$slice::($this,radiusX)}]
-    set data(yRadius) [expr {$y*$slice::($this,radiusY)}]
+    set data(xRadius) [expr {$x*$slice::($this,xRadius)}]
+    set data(yRadius) [expr {$y*$slice::($this,yRadius)}]
     set data(height) [expr {$y*$switched::($this,-height)}]
     foreach {x y} [$slice::($this,canvas) coords $slice::($this,origin)] {}
     set data(xCenter) [expr {$x+$data(xRadius)}]
     set data(yCenter) [expr {$y+$data(yRadius)}]
 }
-
-class slice {
-    proc maximum {a b} {return [expr {$a>$b?$a:$b}]}
-}
-
-set rcsId {$Id: selector.tcl,v 1.1 1998/04/24 10:46:30 jfontain Exp $}
+set rcsId {$Id: selector.tcl,v 1.2 1998/05/24 19:18:21 jfontain Exp $}
 
 
 class selector {
@@ -1448,7 +1466,7 @@ class selector {
     }
 }
 
-set rcsId {$Id: objselec.tcl,v 1.4 1998/04/24 10:51:45 jfontain Exp $}
+set rcsId {$Id: objselec.tcl,v 1.5 1998/05/24 19:24:43 jfontain Exp $}
 
 
 class objectSelector {
@@ -1482,28 +1500,19 @@ class objectSelector {
         return [lsort -integer [selector::_list $this]]
     }
 }
-set rcsId {$Id: pie.tcl,v 1.70 1998/05/03 18:39:41 jfontain Exp $}
+set rcsId {$Id: pie.tcl,v 1.80 1998/06/09 21:20:45 jfontain Exp $}
 
-package provide tkpiechart 5.0
+package provide tkpiechart 5.2
 
 class pie {
     set pie::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
 }
 
-proc pie::pie {this canvas x y width height args} switched {$args} {
+proc pie::pie {this canvas x y args} switched {$args} {
     set pie::($this,canvas) $canvas
-
-    set pie::($this,x) [winfo fpixels $canvas $x]
-    set pie::($this,y) [winfo fpixels $canvas $y]
-    set pie::($this,radiusX) [expr {[winfo fpixels $canvas $width]/2.0}]
-    set pie::($this,radiusY) [expr {[winfo fpixels $canvas $height]/2.0}]
-
     set pie::($this,colorIndex) 0
     set pie::($this,slices) {}
-
-    set pie::($this,xScale) 1
-    set pie::($this,yScale) 1
-
+    set pie::($this,origin) [$canvas create image $x $y -tags pie($this)]
     switched::complete $this
     complete $this
 }
@@ -1513,19 +1522,21 @@ proc pie::~pie {this} {
     delete $pie::($this,labeler)
     eval delete $pie::($this,slices) $pie::($this,backgroundSlice)
     catch {delete $pie::($this,selector)}
+    $pie::($this,canvas) delete $pie::($this,origin)
 }
 
 proc pie::options {this} {
     return [list\
         [list -background {} {}]\
         [list -colors $pie::(colors) $pie::(colors)]\
+        [list -height 200]\
         [list -labeler 0 0]\
         [list -selectable 0 0]\
         [list -thickness 0]\
         [list -title {} {}]\
-        [list -titlefont {} {}]\
+        [list -titlefont {Helvetica -12 bold} {Helvetica -12 bold}]\
         [list -titleoffset 2 2]\
-        [list -scale {1 1} {1 1}]\
+        [list -width 200]\
     ]
 }
 
@@ -1537,18 +1548,6 @@ foreach option {-background -colors -labeler -selectable -title -titlefont -titl
     "
 }
 
-proc pie::set-scale {this value} {
-    foreach {xScale yScale} $value {}
-    if {($xScale==$pie::($this,xScale))&&($yScale==$pie::($this,yScale))} return
-    switched::configure $pie::($this,backgroundSlice) -scale $value
-    foreach slice $pie::($this,slices) {
-        switched::configure $slice -scale $value
-    }
-    pieLabeler::update $pie::($this,labeler)
-    set pie::($this,xScale) $xScale
-    set pie::($this,yScale) $yScale
-}
-
 proc pie::set-thickness {this value} {
     if {$switched::($this,complete)} {
         error {option -thickness cannot be set dynamically}
@@ -1556,6 +1555,23 @@ proc pie::set-thickness {this value} {
     set pie::($this,thickness) [winfo fpixels $pie::($this,canvas) $value]
 }
 
+proc pie::set-height {this value} {
+    set pie::($this,height) [expr {[winfo fpixels $pie::($this,canvas) $value]-1}]
+    if {$switched::($this,complete)} {
+        update $this
+    } else {
+        set pie::($this,initialHeight) $pie::($this,height)
+    }
+}
+proc pie::set-width {this value} {
+    set pie::($this,width) [expr {[winfo fpixels $pie::($this,canvas) $value]-1}]
+    if {$switched::($this,complete)} {
+        update $this
+    } else {
+        set pie::($this,initialWidth) $pie::($this,width)
+    }
+}
+
 proc pie::complete {this} {
     set canvas $pie::($this,canvas)
 
@@ -1565,7 +1581,6 @@ proc pie::complete {this} {
         set pie::($this,labeler) $switched::($this,-labeler)
     }
     $canvas addtag pie($this) withtag pieLabeler($pie::($this,labeler))
-    pieLabeler::link $pie::($this,labeler) $this
 
     if {[string length $switched::($this,-background)]==0} {
         set bottomColor {}
@@ -1573,12 +1588,23 @@ proc pie::complete {this} {
         set bottomColor [tkDarken $switched::($this,-background) 60]
     }
     set slice [new slice\
-        $canvas $pie::($this,x) $pie::($this,y) $pie::($this,radiusX) $pie::($this,radiusY) 90 360\
-        -height $pie::($this,thickness) -topcolor $switched::($this,-background) -bottomcolor $bottomColor\
+        $canvas [expr {$pie::($this,initialWidth)/2}] [expr {$pie::($this,initialHeight)/2}]\
+        -startandextent {90 360} -height $pie::($this,thickness) -topcolor $switched::($this,-background) -bottomcolor $bottomColor\
     ]
     $canvas addtag pie($this) withtag slice($slice)
-    $canvas addtag pieGraphics($this) withtag slice($slice)
+    $canvas addtag pieSlices($this) withtag slice($slice)
     set pie::($this,backgroundSlice) $slice
+    if {[string length $switched::($this,-title)]==0} {
+        set pie::($this,titleRoom) 0
+    } else {
+        set pie::($this,title) [$canvas create text 0 0\
+            -anchor n -text $switched::($this,-title) -font $switched::($this,-titlefont) -tags pie($this)\
+        ]
+        set pie::($this,titleRoom) [expr {\
+            [font metrics $switched::($this,-titlefont) -ascent]+[winfo fpixels $canvas $switched::($this,-titleoffset)]\
+        }]
+    }
+    update $this
 }
 
 proc pie::newSlice {this {text {}}} {
@@ -1591,14 +1617,13 @@ proc pie::newSlice {this {text {}}} {
     set color [lindex $switched::($this,-colors) $pie::($this,colorIndex)]
     set pie::($this,colorIndex) [expr {($pie::($this,colorIndex)+1)%[llength $switched::($this,-colors)]}]
 
-    foreach {x y} [$canvas coords slice($pie::($this,backgroundSlice))] {}
-
     set slice [new slice\
-        $canvas $x $y $pie::($this,radiusX) $pie::($this,radiusY) $start 0 -scale $switched::($this,-scale)\
+        $canvas [expr {$pie::($this,initialWidth)/2}] [expr {$pie::($this,initialHeight)/2}] -startandextent "$start 0"\
         -height $pie::($this,thickness) -topcolor $color -bottomcolor [tkDarken $color 60]\
     ]
+    eval $canvas move slice($slice) [$canvas coords pieSlices($this)]
     $canvas addtag pie($this) withtag slice($slice)
-    $canvas addtag pieGraphics($this) withtag slice($slice)
+    $canvas addtag pieSlices($this) withtag slice($slice)
     lappend pie::($this,slices) $slice
 
     if {[string length $text]==0} {
@@ -1609,6 +1634,8 @@ proc pie::newSlice {this {text {}}} {
     set pie::($this,sliceLabel,$slice) $label
     $canvas addtag pie($this) withtag pieLabeler($labeler)
 
+    update $this
+
     if {$switched::($this,-selectable)} {
         if {![info exists pie::($this,selector)]} {
             set pie::($this,selector) [new objectSelector -selectcommand "pie::setLabelsState $this"]
@@ -1642,6 +1669,7 @@ proc pie::deleteSlice {this slice} {
         selector::remove $pie::($this,selector) $pie::($this,sliceLabel,$slice)
     }
     unset pie::($this,sliceLabel,$slice)
+    update $this
 }
 
 proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
@@ -1651,7 +1679,7 @@ proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
     }
     set newExtent [expr {[maximum [minimum $unitShare 1] 0]*360}]
     set growth [expr {$newExtent-$slice::($slice,extent)}]
-    slice::update $slice [expr {$slice::($slice,start)-$growth}] $newExtent
+    switched::configure $slice -startandextent "[expr {$slice::($slice,start)-$growth}] $newExtent"
 
     if {[string length $valueToDisplay]>0} {
         pieLabeler::set $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $valueToDisplay
@@ -1665,21 +1693,6 @@ proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
     }
 }
 
-proc pie::createTitle {this string font offset} {
-    if {[string length $string]==0} {
-        return
-    }
-    set canvas $pie::($this,canvas)
-    set box [$canvas bbox pie($this)]
-    set pie::($this,title) [\
-        $canvas create text [expr {[lindex $box 0]+([lindex $box 2]-[lindex $box 0])/2}] [expr {[lindex $box 1]-$offset}]\
-            -anchor s -tags pie($this) -text $string\
-    ]
-    if {[string length $font]>0} {
-        $canvas itemconfigure $pie::($this,title) -font $font
-    }
-}
-
 proc pie::selectedSlices {this} {
     set list {}
     foreach slice $pie::($this,slices) {
@@ -1697,6 +1710,44 @@ proc pie::setLabelsState {this labels selected} {
     }
 }
 
+proc pie::currentSlice {this} {
+    set tags [$pie::($this,canvas) gettags current]
+    if {[scan $tags slice(%u) slice]>0} {
+        return $slice
+    }
+    if {[scan $tags canvasLabel(%u) label]>0} {
+        foreach slice $pie::($this,slices) {
+            if {$pie::($this,sliceLabel,$slice)==$label} {
+                return $slice
+            }
+        }
+    }
+    return 0
+}
+
+proc pie::update {this} {
+    set canvas $pie::($this,canvas)
+    pieLabeler::room $pie::($this,labeler) room
+    foreach {x y} [$canvas coords $pie::($this,origin)] {}
+    foreach {xSlices ySlices} [$canvas coords pieSlices($this)] {}
+    $canvas move pieSlices($this) [expr {$x+$room(left)-$xSlices}] [expr {$y+$room(top)+$pie::($this,titleRoom)-$ySlices}]
+    set scale [list\
+        [expr {($pie::($this,width)-$room(left)-$room(right))/$pie::($this,initialWidth)}]\
+        [expr {\
+            ($pie::($this,height)-$room(top)-$room(bottom)-$pie::($this,titleRoom))/\
+            ($pie::($this,initialHeight)+$pie::($this,thickness))\
+        }]\
+    ]
+    switched::configure $pie::($this,backgroundSlice) -scale $scale
+    foreach slice $pie::($this,slices) {
+        switched::configure $slice -scale $scale
+    }
+    if {$pie::($this,titleRoom)>0} {
+        $canvas coords $pie::($this,title) [expr {$x+($pie::($this,width)/2)}] $y
+    }
+    pieLabeler::update $pie::($this,labeler) $x $y [expr {$x+$pie::($this,width)}] [expr {$y+$pie::($this,height)}]
+}
+
 class pie {
     proc maximum {a b} {return [expr {$a>$b?$a:$b}]}
     proc minimum {a b} {return [expr {$a<$b?$a:$b}]}
@@ -1711,9 +1762,9 @@ set canvas [canvas .c -highlightthickness 0]
 pack $canvas -fill both -expand 1
 
 set pie1 [new pie\
-    $canvas 0 0 200 100 -thickness 20 -background gray\
+    $canvas 0 0 -height 100 -thickness 20 -background gray\
     -labeler [new pieBoxLabeler $canvas -justify center -offset 10]\
-    -title "this is pie #1" -titlefont fixed -selectable 1\
+    -title "this is pie #1" -titlefont fixed -titleoffset 6 -selectable 1\
 ]
 set slice11 [pie::newSlice $pie1]
 set slice12 [pie::newSlice $pie1]
@@ -1721,12 +1772,12 @@ set slice13 [pie::newSlice $pie1]
 set slice14 [pie::newSlice $pie1 {some text}]
 
 set pie2 [new pie\
-    $canvas 0 0 200 100 -thickness 10 -background white\
+    $canvas 0 0 -height 100 -thickness 10 -background white\
     -labeler [\
         new piePeripheralLabeler $canvas -font {-weight bold -family Helvetica -size -20}\
-            -smallfont {-family Helvetica -size -8} -bulletwidth 1c -xoffset 1\
+            -smallfont {-family Helvetica -size -8} -bulletwidth 1c\
     ]\
-    -title "this is pie #2" -titleoffset 20\
+    -title "this is pie #2" -titleoffset 10\
 ]
 set slice21 [pie::newSlice $pie2]
 set slice22 [pie::newSlice $pie2]
@@ -1781,20 +1832,15 @@ proc refresh {} {
     after 3000 refresh
 }
 
-set initialWidth 450
-set initialHeight 250
-
 proc resize {width height} {
-    set scale [list\
-        [expr {double($width)/$::initialWidth}]\
-        [expr {double($height)/$::initialHeight}]\
-    ]
-    switched::configure $::pie1 -scale $scale
-    switched::configure $::pie2 -scale $scale
+    set width [expr {$width/2.0}]
+    set height [expr {$height/2.0}]
+    switched::configure $::pie1 -width $width -height $height
+    switched::configure $::pie2 -width $width -height $height
     $::canvas configure -scrollregion [$::canvas bbox all]
 }
 
-$canvas configure -width $initialWidth -height $initialHeight
+$canvas configure -width 400 -height 300
 bind $canvas <Configure> "resize %w %h"
 refresh
 
