@@ -27,7 +27,7 @@ if {[catch {package require stooop 3.5} message]} {
     exit 1
 }
 
-if {[catch {eval file copy -force {pkgIndex.tcl pie.tcl slice.tcl pielabel.tcl boxlabel.tcl canlabel.tcl perilabel.tcl labarray.tcl objselec.tcl} {$directory}} message]} {
+if {[catch {eval file copy -force {pkgIndex.tcl pie.tcl slice.tcl pielabel.tcl boxlabel.tcl canlabel.tcl perilabel.tcl labarray.tcl selector.tcl objselec.tcl} {$directory}} message]} {
     puts stderr $message
     exit 1
 }
