@@ -1,4 +1,4 @@
-set rcsId {$Id: perilabel.tcl,v 1.39 1998/05/03 18:15:53 jfontain Exp $}
+set rcsId {$Id: perilabel.tcl,v 1.45 1998/06/09 21:17:52 jfontain Exp $}
 
 class piePeripheralLabeler {
 
@@ -6,6 +6,10 @@ class piePeripheralLabeler {
 
     proc piePeripheralLabeler {this canvas args} pieLabeler {$canvas $args} switched {$args} {
         switched::complete $this
+        ::set piePeripheralLabeler::($this,array) [::new canvasLabelsArray $canvas -justify $switched::($this,-justify)]
+        ::set piePeripheralLabeler::($this,valueWidth)\
+            [font measure $switched::($this,-smallfont) $switched::($this,-widestvaluetext)]
+        ::set piePeripheralLabeler::($this,valueHeight) [font metrics $switched::($this,-smallfont) -ascent]
     }
 
     proc ~piePeripheralLabeler {this} {
@@ -21,11 +25,11 @@ class piePeripheralLabeler {
             [list -justify left left]\
             [list -offset 5 5]\
             [list -smallfont {Helvetica -10} {Helvetica -10}]\
-            [list -xoffset 0 0]\
+            [list -widestvaluetext 0.00 0.00]\
         ]
     }
 
-    foreach option {-bulletwidth -font -justify -offset -smallfont -xoffset} {                         ;# no dynamic options allowed
+    foreach option {-bulletwidth -font -justify -offset -smallfont -widestvaluetext} {                 ;# no dynamic options allowed
         proc set$option {this value} "
             if {\$switched::(\$this,complete)} {
                 error {option $option cannot be set dynamically}
@@ -33,32 +37,26 @@ class piePeripheralLabeler {
         "
     }
 
+    proc set-smallfont {this value} {
+        if {$switched::($this,complete)} {
+            error {option -smallfont cannot be set dynamically}
+        }
+    }
+
     proc new {this slice args} {                                       ;# variable arguments are for the created canvas label object
         ::set canvas $pieLabeler::($this,canvas)
-
         ::set text [$canvas create text 0 0 -font $switched::($this,-smallfont) -tags pieLabeler($this)]       ;# create value label
-        ::set box [$canvas bbox $text]
-        ::set smallTextHeight [expr {[lindex $box 3]-[lindex $box 1]}]
-
-        if {![info exists piePeripheralLabeler::($this,array)]} {                                     ;# create a split labels array
-            ::set piePeripheralLabeler::($this,array)\
-                [::new canvasLabelsArray $canvas 0 0 -justify $switched::($this,-justify) -xoffset $switched::($this,-xoffset)]
-            update $this                                                                                 ;# position array below pie
-        }
-        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas) 0 0 $args\
+        ::set label [eval ::new canvasLabel $pieLabeler::($this,canvas) $args\
             [list\
                 -style split -justify $switched::($this,-justify) -bulletwidth $switched::($this,-bulletwidth)\
                 -font $switched::($this,-font)\
             ]\
         ]
         canvasLabelsArray::manage $piePeripheralLabeler::($this,array) $label
-
         $canvas addtag pieLabeler($this) withtag canvasLabelsArray($piePeripheralLabeler::($this,array))         ;# refresh our tags
-
         ::set piePeripheralLabeler::($this,textItem,$label) $text                       ;# value text item is the only one to update
         ::set piePeripheralLabeler::($this,slice,$label) $slice
         ::set piePeripheralLabeler::($this,selected,$label) 0
-
         return $label
     }
 
@@ -123,18 +121,29 @@ class piePeripheralLabeler {
         ::set piePeripheralLabeler::($this,selected,$label) $selected
     }
 
-    proc update {this} {
+    proc update {this left top right bottom} {                                   ;# whole pie coordinates, includings labeler labels
         ::set canvas $pieLabeler::($this,canvas)
-        ::set box [$canvas bbox pieGraphics($pieLabeler::($this,pie))]
-::set smallTextHeight 10 ;########### calculate in set-smallfont{} #################
         ::set array $piePeripheralLabeler::($this,array)                         ;# first reposition labels array below pie graphics
         foreach {x y} [$canvas coords canvasLabelsArray($array)] {}
-        $canvas move canvasLabelsArray($array) [expr {[lindex $box 0]-$x}]\
-            [expr {[lindex $box 3]+(2*$switched::($this,-offset))+$smallTextHeight-$y}]
-        switched::configure $array -width [expr {[lindex $box 2]-[lindex $box 0]}]                             ;# then fit pie width
+        $canvas move canvasLabelsArray($array) [expr {$left-$x}] [expr {$bottom-[canvasLabelsArray::height $array]-$y}]
+        switched::configure $array -width [expr {$right-$left}]                                                ;# then fit pie width
         foreach label [canvasLabelsArray::labels $array] {                                   ;# finally reposition peripheral labels
             position $this $piePeripheralLabeler::($this,textItem,$label) $piePeripheralLabeler::($this,slice,$label)
         }
     }
 
+    proc room {this arrayName} {
+        upvar $arrayName data
+
+        ::set data(left) [expr {$piePeripheralLabeler::($this,valueWidth)+$switched::($this,-offset)}]
+        ::set data(right) $data(left)
+        ::set data(top) [expr {$switched::($this,-offset)+$piePeripheralLabeler::($this,valueHeight)}]
+        ::set box [$pieLabeler::($this,canvas) bbox canvasLabelsArray($piePeripheralLabeler::($this,array))]
+        if {[llength $box]==0} {                                                                                    ;# no labels yet
+            ::set data(bottom) $data(top)
+        } else {                                                                        ;# room taken by all labels including offset
+            ::set data(bottom) [expr {$data(top)+[lindex $box 3]-[lindex $box 1]}]
+        }
+    }
+
 }
