@@ -1,25 +1,17 @@
-set rcsId {$Id: pie.tcl,v 1.70 1998/05/03 18:39:41 jfontain Exp $}
+set rcsId {$Id: pie.tcl,v 1.80 1998/06/09 21:20:45 jfontain Exp $}
 
-package provide tkpiechart 5.0
+package provide tkpiechart 5.2
 
 class pie {
     set pie::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
 }
 
-proc pie::pie {this canvas x y width height args} switched {$args} {            ;# note: all pie elements are tagged with pie($this)
+proc pie::pie {this canvas x y args} switched {$args} {                         ;# note: all pie elements are tagged with pie($this)
     set pie::($this,canvas) $canvas
-
-    set pie::($this,x) [winfo fpixels $canvas $x]
-    set pie::($this,y) [winfo fpixels $canvas $y]
-    set pie::($this,radiusX) [expr {[winfo fpixels $canvas $width]/2.0}]
-    set pie::($this,radiusY) [expr {[winfo fpixels $canvas $height]/2.0}]
-
     set pie::($this,colorIndex) 0
     set pie::($this,slices) {}
-
-    set pie::($this,xScale) 1
-    set pie::($this,yScale) 1
-
+    # use an empty image as an origin marker with only 2 coordinates
+    set pie::($this,origin) [$canvas create image $x $y -tags pie($this)]
     switched::complete $this
     complete $this                                                  ;# wait till all options have been set for initial configuration
 }
@@ -29,20 +21,22 @@ proc pie::~pie {this} {
     delete $pie::($this,labeler)
     eval delete $pie::($this,slices) $pie::($this,backgroundSlice)
     catch {delete $pie::($this,selector)}
+    $pie::($this,canvas) delete $pie::($this,origin)
 }
 
 proc pie::options {this} {
-    # force thickness option so that corresponding pie member is properly initialized
+    # force height, thickness title font and width options so that corresponding members are properly initialized
     return [list\
         [list -background {} {}]\
         [list -colors $pie::(colors) $pie::(colors)]\
+        [list -height 200]\
         [list -labeler 0 0]\
         [list -selectable 0 0]\
         [list -thickness 0]\
         [list -title {} {}]\
-        [list -titlefont {} {}]\
+        [list -titlefont {Helvetica -12 bold} {Helvetica -12 bold}]\
         [list -titleoffset 2 2]\
-        [list -scale {1 1} {1 1}]\
+        [list -width 200]\
     ]
 }
 
@@ -55,18 +49,6 @@ foreach option {-background -colors -labeler -selectable -title -titlefont -titl
     "
 }
 
-proc pie::set-scale {this value} {
-    foreach {xScale yScale} $value {}
-    if {($xScale==$pie::($this,xScale))&&($yScale==$pie::($this,yScale))} return                                        ;# no change
-    switched::configure $pie::($this,backgroundSlice) -scale $value                             ;# update scale of background slice,
-    foreach slice $pie::($this,slices) {
-        switched::configure $slice -scale $value                                                                 ;# and other slices
-    }
-    pieLabeler::update $pie::($this,labeler)                          ;# finally update labels now that pie graphics are in position
-    set pie::($this,xScale) $xScale
-    set pie::($this,yScale) $yScale
-}
-
 proc pie::set-thickness {this value} {
     if {$switched::($this,complete)} {
         error {option -thickness cannot be set dynamically}
@@ -74,7 +56,25 @@ proc pie::set-thickness {this value} {
     set pie::($this,thickness) [winfo fpixels $pie::($this,canvas) $value]                                      ;# convert to pixels
 }
 
-proc pie::complete {this} {
+# size is first converted to pixels, then 1 pixel is subtracted since slice size is half the pie size and pie center takes 1 pixel
+proc pie::set-height {this value} {                                                ;# height is slices height not counting thickness
+    set pie::($this,height) [expr {[winfo fpixels $pie::($this,canvas) $value]-1}]
+    if {$switched::($this,complete)} {
+        update $this
+    } else {
+        set pie::($this,initialHeight) $pie::($this,height)           ;# keep track of initial value for latter scaling calculations
+    }
+}
+proc pie::set-width {this value} {
+    set pie::($this,width) [expr {[winfo fpixels $pie::($this,canvas) $value]-1}]
+    if {$switched::($this,complete)} {
+        update $this
+    } else {
+        set pie::($this,initialWidth) $pie::($this,width)             ;# keep track of initial value for latter scaling calculations
+    }
+}
+
+proc pie::complete {this} {                                                                              ;# no user slices exist yet
     set canvas $pie::($this,canvas)
 
     if {$switched::($this,-labeler)==0} {
@@ -83,7 +83,6 @@ proc pie::complete {this} {
         set pie::($this,labeler) $switched::($this,-labeler)                                             ;# use user defined labeler
     }
     $canvas addtag pie($this) withtag pieLabeler($pie::($this,labeler))
-    pieLabeler::link $pie::($this,labeler) $this                                                            ;# link labeler with pie
 
     if {[string length $switched::($this,-background)]==0} {
         set bottomColor {}
@@ -91,12 +90,23 @@ proc pie::complete {this} {
         set bottomColor [tkDarken $switched::($this,-background) 60]
     }
     set slice [new slice\
-        $canvas $pie::($this,x) $pie::($this,y) $pie::($this,radiusX) $pie::($this,radiusY) 90 360\
-        -height $pie::($this,thickness) -topcolor $switched::($this,-background) -bottomcolor $bottomColor\
+        $canvas [expr {$pie::($this,initialWidth)/2}] [expr {$pie::($this,initialHeight)/2}]\
+        -startandextent {90 360} -height $pie::($this,thickness) -topcolor $switched::($this,-background) -bottomcolor $bottomColor\
     ]
     $canvas addtag pie($this) withtag slice($slice)
-    $canvas addtag pieGraphics($this) withtag slice($slice)
+    $canvas addtag pieSlices($this) withtag slice($slice)
     set pie::($this,backgroundSlice) $slice
+    if {[string length $switched::($this,-title)]==0} {
+        set pie::($this,titleRoom) 0
+    } else {
+        set pie::($this,title) [$canvas create text 0 0\
+            -anchor n -text $switched::($this,-title) -font $switched::($this,-titlefont) -tags pie($this)\
+        ]
+        set pie::($this,titleRoom) [expr {\
+            [font metrics $switched::($this,-titlefont) -ascent]+[winfo fpixels $canvas $switched::($this,-titleoffset)]\
+        }]
+    }
+    update $this
 }
 
 proc pie::newSlice {this {text {}}} {
@@ -109,16 +119,14 @@ proc pie::newSlice {this {text {}}} {
     set color [lindex $switched::($this,-colors) $pie::($this,colorIndex)]                                        ;# get a new color
     set pie::($this,colorIndex) [expr {($pie::($this,colorIndex)+1)%[llength $switched::($this,-colors)]}]  ;# circle through colors
 
-    # make sure slice is positioned correctly in case pie was moved
-    foreach {x y} [$canvas coords slice($pie::($this,backgroundSlice))] {}
-
     # darken slice top color by 40% to obtain bottom color, as it is done for Tk buttons shadow, for example
     set slice [new slice\
-        $canvas $x $y $pie::($this,radiusX) $pie::($this,radiusY) $start 0 -scale $switched::($this,-scale)\
+        $canvas [expr {$pie::($this,initialWidth)/2}] [expr {$pie::($this,initialHeight)/2}] -startandextent "$start 0"\
         -height $pie::($this,thickness) -topcolor $color -bottomcolor [tkDarken $color 60]\
     ]
+    eval $canvas move slice($slice) [$canvas coords pieSlices($this)]  ;# place slice at other slices position in case pie was moved
     $canvas addtag pie($this) withtag slice($slice)
-    $canvas addtag pieGraphics($this) withtag slice($slice)
+    $canvas addtag pieSlices($this) withtag slice($slice)
     lappend pie::($this,slices) $slice
 
     if {[string length $text]==0} {                                                           ;# generate label text if not provided
@@ -129,6 +137,8 @@ proc pie::newSlice {this {text {}}} {
     set pie::($this,sliceLabel,$slice) $label
     $canvas addtag pie($this) withtag pieLabeler($labeler)                     ;# update tags which canvas does not automatically do
 
+    update $this
+
     if {$switched::($this,-selectable)} {                                             ;# toggle select state at every button release
         if {![info exists pie::($this,selector)]} {                                                  ;# create selector if necessary
             set pie::($this,selector) [new objectSelector -selectcommand "pie::setLabelsState $this"]
@@ -163,6 +173,7 @@ proc pie::deleteSlice {this slice} {
         selector::remove $pie::($this,selector) $pie::($this,sliceLabel,$slice)
     }
     unset pie::($this,sliceLabel,$slice)
+    update $this
 }
 
 proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
@@ -173,7 +184,7 @@ proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
     # cannot display slices that occupy more than whole pie and less than zero
     set newExtent [expr {[maximum [minimum $unitShare 1] 0]*360}]
     set growth [expr {$newExtent-$slice::($slice,extent)}]
-    slice::update $slice [expr {$slice::($slice,start)-$growth}] $newExtent                                        ;# grow clockwise
+    switched::configure $slice -startandextent "[expr {$slice::($slice,start)-$growth}] $newExtent"                ;# grow clockwise
 
     if {[string length $valueToDisplay]>0} {                  ;# update label after slice for it may need slice latest configuration
         pieLabeler::set $pie::($this,labeler) $pie::($this,sliceLabel,$slice) $valueToDisplay
@@ -187,21 +198,6 @@ proc pie::sizeSlice {this slice unitShare {valueToDisplay {}}} {
     }
 }
 
-proc pie::createTitle {this string font offset} {
-    if {[string length $string]==0} {
-        return
-    }
-    set canvas $pie::($this,canvas)                                                                      ;# place text on top of pie
-    set box [$canvas bbox pie($this)]
-    set pie::($this,title) [\
-        $canvas create text [expr {[lindex $box 0]+([lindex $box 2]-[lindex $box 0])/2}] [expr {[lindex $box 1]-$offset}]\
-            -anchor s -tags pie($this) -text $string\
-    ]
-    if {[string length $font]>0} {
-        $canvas itemconfigure $pie::($this,title) -font $font
-    }
-}
-
 proc pie::selectedSlices {this} {                                                      ;# return a list of currently selected slices
     set list {}
     foreach slice $pie::($this,slices) {
@@ -219,6 +215,45 @@ proc pie::setLabelsState {this labels selected} {
     }
 }
 
+proc pie::currentSlice {this} {                           ;# return current slice (slice or its label under the mouse cursor) if any
+    set tags [$pie::($this,canvas) gettags current]
+    if {[scan $tags slice(%u) slice]>0} {
+        return $slice                                                                                         ;# found current slice
+    }
+    if {[scan $tags canvasLabel(%u) label]>0} {
+        foreach slice $pie::($this,slices) {
+            if {$pie::($this,sliceLabel,$slice)==$label} {
+                return $slice                                                                  ;# slice is current through its label
+            }
+        }
+    }
+    return 0                                                                                                     ;# no current slice
+}
+
+proc pie::update {this} {                         ;# place and scale slices along and with labels array in its current configuration
+    set canvas $pie::($this,canvas)
+    pieLabeler::room $pie::($this,labeler) room                                                     ;# take labels room into account
+    foreach {x y} [$canvas coords $pie::($this,origin)] {}                                       ;# retrieve current pie coordinates
+    foreach {xSlices ySlices} [$canvas coords pieSlices($this)] {}                  ;# move slices in order to leave room for labels
+    $canvas move pieSlices($this) [expr {$x+$room(left)-$xSlices}] [expr {$y+$room(top)+$pie::($this,titleRoom)-$ySlices}]
+    set scale [list\
+        [expr {($pie::($this,width)-$room(left)-$room(right))/$pie::($this,initialWidth)}]\
+        [expr {\
+            ($pie::($this,height)-$room(top)-$room(bottom)-$pie::($this,titleRoom))/\
+            ($pie::($this,initialHeight)+$pie::($this,thickness))\
+        }]\
+    ]
+    switched::configure $pie::($this,backgroundSlice) -scale $scale                             ;# update scale of background slice,
+    foreach slice $pie::($this,slices) {
+        switched::configure $slice -scale $scale                                                                 ;# and other slices
+    }
+    if {$pie::($this,titleRoom)>0} {                                                                                 ;# title exists
+        $canvas coords $pie::($this,title) [expr {$x+($pie::($this,width)/2)}] $y               ;# place text above pie and centered
+    }
+    # finally update labels now that pie graphics are in position
+    pieLabeler::update $pie::($this,labeler) $x $y [expr {$x+$pie::($this,width)}] [expr {$y+$pie::($this,height)}]
+}
+
 class pie {                                                                                     ;# define various utility procedures
     proc maximum {a b} {return [expr {$a>$b?$a:$b}]}
     proc minimum {a b} {return [expr {$a<$b?$a:$b}]}
