@@ -1,6 +1,6 @@
-set rcsId {$Id: stooop.tcl,v 3.60 1998/04/18 20:43:14 jfontain Exp $}
+set rcsId {$Id: stooop.tcl,v 3.62 1998/06/05 18:29:09 jfontain Exp $}
 
-package provide stooop 3.5.1
+package provide stooop 3.6
 
 catch {rename proc _proc}            ;# rename proc before it is overloaded, ignore error in case of multiple inclusion of this file
 
@@ -368,7 +368,7 @@ _proc ::stooop::memberProcedureDeclaration {fullClass class procedure arguments
             _proc ${fullClass}::$procedure $arguments \
 "::variable {}
 $checkCode
-::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         } else {                                                                                      ;# regular virtual declaration
             # setup access to class data
@@ -383,9 +383,9 @@ $body
 "::variable {}
 $checkCode
 if {!\[::catch {::info body \$${fullClass}::(\$this,_derived)::$procedure}\]} {
-::return \[::eval \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
+::return \[::uplevel \$${fullClass}::(\$this,_derived)::$procedure \[::lrange \[::info level 0\] 1 end\]\]
 }
-::eval ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
+::uplevel ${fullClass}::_$procedure \[::lrange \[::info level 0\] 1 end\]
 "
         }
     } else {                                                                                              ;# non virtual declaration
